from lib2to3 import fixer_base
from lib2to3.pytree import Leaf
from lib2to3.fixer_util import token, syms, String, Newline, Name, Assign
from ..fixer_util import indentation, Node, syms, paste_after

class _FixDecoratorBase(fixer_base.BaseFix):

    _PATTERN = """
               decorated < one_dec=decorator < any* > def={0} < ('class'|'def') name=any any* > > | 
               decorated < decorators < decs=decorator+ > def={0} < ('class'|'def') name=any any* > >
               """
    def _fix_dec(self, oldname, newname, dec):
        dotted_name_node = dec.children[1]
        if isinstance(dotted_name_node, Leaf):
            name_ref_leaf = dotted_name_node
        else:
            name_ref_leaf = dotted_name_node.children[0]
        if name_ref_leaf.value == oldname:
            name_ref_leaf.value = newname
            return True
        return False

    def transform(self, node, results):
        codedef = results["def"]
        node_idx = node.parent.children.index(node)
        decs = [results["one_dec"]] if results.get("one_dec") is not None else results["decs"]
        func_name = codedef.children[1].value
        tmp_name = "_lib3to2_tmp_dec"
        rename_assign = Assign(Name(tmp_name), Name(func_name))
        rename_line = Node(syms.simple_stmt, [rename_assign, Newline()])
        rename_needed = False
        for dec in decs:
            rename_needed = rename_needed or self._fix_dec(func_name, tmp_name, dec)
        dec_strings = [str(dec).strip()[1:] for dec in decs]
        assign = ""
        for dec in dec_strings:
            assign += dec
            assign += "("
        assign += results["name"].value
        for dec in dec_strings:
            assign += ")"
        assign = String(results["name"].value + " = " + assign)
        
        del_part = []
        if rename_needed:
            del_stmt = Node(syms.del_stmt, [Name("del"), Name(tmp_name, prefix=" ")])
            del_stmt.prefix = " "
            del_part = [Leaf(token.SEMI, ";"), del_stmt]
            
        assign_statement = Node(syms.simple_stmt, [assign] + del_part + [Newline()])

        codedef.prefix = node.prefix
        node.replace(codedef)
        
        if rename_needed:
            rename_line.prefix = codedef.prefix
            codedef.parent.insert_child(node_idx, rename_line)
            codedef.prefix = indentation(rename_line)
        
        paste_after(codedef, assign_statement)