"""
Fixer for bytes -> str.
"""

import re
from lib2to3 import fixer_base
from lib2to3.patcomp import compile_pattern
from ..fixer_util import Name, token, syms, parse_args, Call, Comma

class FixBytes(fixer_base.BaseFix):
    
    order = "pre"
    run_order = 9
    
    PATTERN = "power< 'bytes' [trailer< '(' (args=arglist | any*) ')' >] > | 'bytes'"

    def transform(self, node, results):
        name = results.get("name")
        arglist = results.get("args")
        if node.type == token.NAME:
            return Name("str", prefix=node.prefix)
        
        if arglist is not None:
            args = arglist.children
            parsed = parse_args(args, ("source", "encoding", "errors"))

            source, encoding, errors = (parsed[v] for v in ("source", "encoding", "errors"))
            encoding.prefix = ""
            str_call = Call(Name("str"), ([source.clone()]))
            if errors is None:
                node.replace(Call(Name(str(str_call) + ".encode"), (encoding.clone(),)))
            else:
                errors.prefix = " "
                node.replace(Call(Name(str(str_call) + ".encode"), (encoding.clone(), Comma(), errors.clone())))
