"""
Fixer for:
it.__next__() -> it.next().
next(it) -> it.next().
"""

from lib2to3.pgen2 import token
from lib2to3.pygram import python_symbols as syms
from lib2to3 import fixer_base
from lib2to3.fixer_util import Name, Call, find_binding, Attr, parenthesize
from lib2to3.pytree import Node

bind_warning = "Calls to builtin next() possibly shadowed by global binding"


class FixNext(fixer_base.BaseFix):
    
    order = 'pre'
    run_order = 4

    PATTERN = """
    power< base=any+ trailer< '.' attr='__next__' > any* >
    |
    power< head='next' trailer< '(' arg=any ')' > any* >
    |
    classdef< 'class' base=any+ ':'
              suite< any*
                     funcdef< 'def'
                              attr='__next__'
                              parameters< '(' NAME ')' > any+ >
                     any* > >
    """

    def transform(self, node, results):
        assert results

        base = results.get("base")
        attr = results.get("attr")
        head = results.get("head")
        arg_ = results.get("arg")
        
        if arg_:
            arg = arg_.clone()
            if arg.type == syms.arglist:
                self.warning(node, "Cannot convert next() with "
                                   "default= keyword argument.")
                return
            elif (isinstance(arg, Node) and len(arg.children) == 2
                  and arg.children[1].type == syms.comp_for):  
                # unenclosed generator expression
                arg.type = syms.testlist_gexp
                next_base = parenthesize(arg)
            else:
                next_base = Name(str(arg))
            next_base.prefix = head.prefix
            head.replace(Attr(next_base, Name('next')))
            arg_.remove()
        elif base:
            attr.replace(Name("next", prefix=attr.prefix))
