"""Fixer for 'raise E(V).with_traceback(T)' -> 'raise E, V, T'"""

from lib2to3 import fixer_base
from ..fixer_util import Comma, Node, Leaf, token, syms
from ..fixer_util import Assign, Name, Attr

class FixRaise(fixer_base.BaseFix):

    PATTERN = """
    raise_stmt< 'raise' exc=(power< name=any [trailer< '(' val=any* ')' >]
        [trailer< '.' 'with_traceback' > trailer< '(' trc=any ')' >] > | any) ['from' chain=any] >"""

    def transform(self, node, results):
        name, val, trc = (results.get("name"), results.get("val"), results.get("trc"))
        chain = results.get("chain")
        if trc is not None:
            if chain is not None:
                self.cannot_convert(node, 
                  'Exception chaining and custom traceback are not'
                  'supported if used together.')
                chain.prev_sibling.remove()
                chain.remove()
            val = val[0] if val else Leaf(token.NAME, "None")
            val.prefix = trc.prefix = " "
            kids = [Leaf(token.NAME, "raise"), name.clone(), Comma(),
                    val.clone(), Comma(), trc.clone()]
            raise_stmt = Node(syms.raise_stmt, kids)
            node.replace(raise_stmt)
        elif chain is not None:
            exc = results['exc'][0]
            parent = node.parent
            assert parent.type == syms.simple_stmt
            tmp_var_name = self.new_name('_3to2_exc')
            tmp_var_assign = Assign(Name(tmp_var_name), exc.clone())
            tmp_var_attr = Attr(Name(tmp_var_name), Name('__cause__'))
            exc_cause_assign = Assign(tmp_var_attr, chain.clone())
            raise_stmt = Node(syms.raise_stmt, 
                              [Name('raise'), Name(tmp_var_name, prefix=' ')])
            raise_stmt.prefix = exc_cause_assign.prefix = ' '
            # replace and insert inline to containing simple_stmt
            node.replace(raise_stmt)
            node_idx = parent.children.index(raise_stmt)
            if node_idx > 0:
                assert node_idx - 1 >= 0
                assert parent.children[node_idx-1].type == token.SEMI
                tmp_var_assign.prefix = ' '
            semicolon = lambda: Leaf(token.SEMI, ';')
            parent.insert_child(node_idx, semicolon())
            parent.insert_child(node_idx, exc_cause_assign)
            parent.insert_child(node_idx, semicolon())       
            parent.insert_child(node_idx, tmp_var_assign)
