"""
Fixer for:

class Foo:
    def something(self):
        super()

->

class Foo:
    def something(self):
        super(Foo, self)
"""

from lib2to3 import fixer_base
from lib2to3.fixer_util import token, syms, Name, Comma

def get_firstparam(node):
    children = node.children
    assert len(children) > 2
    params = children[2]
    assert params.type == syms.parameters
    if len(params.children) < 3:
        # Function has no parameters, therefore super() makes no sense here...
        return None
    args = params.children[1]
    if args.type == token.NAME:
        return args.value
    elif args.type == syms.typedargslist:
        assert len(args.children) > 0
        if args.children[0].type == token.NAME:
            return args.children[0].value
        else:
            # Probably a '*'
            return None

def insert_args(class_name, instance_name, rparen):
    parent = rparen.parent
    idx = parent.children.index(rparen)
    parent.insert_child(idx, Name(instance_name, prefix=" "))
    parent.insert_child(idx, Comma())
    parent.insert_child(idx, Name(class_name))

def _find_parent(p_type, node):
    parent = node.parent
    while parent.type != p_type and parent.parent:
        parent = parent.parent
    
    if parent.type != p_type:
        # super() called without arguments outside of a funcdef
        return None
    return parent

find_funcdef = lambda node: _find_parent(syms.funcdef, node)
find_classdef = lambda node: _find_parent(syms.classdef, node)

class FixSuper(fixer_base.BaseFix):

    PATTERN = "power< 'super' trailer< '(' rparen=')' > any* >"

    def transform(self, node, results):
        funcdef_node = find_funcdef(node)
        instance_name = (get_firstparam(funcdef_node) 
                         if funcdef_node is not None
                         else None)
        if instance_name is None:
            self.cannot_convert(node, 
              "super() with no arguments must be called inside"
              "a function that has at least one parameter.")
            return
        
        classdef_node = find_classdef(funcdef_node)
        
        if classdef_node is None:
            self.cannot_convert(node,
              "super() must be called within class definition.")
        
        class_name = classdef_node.children[1].value 
        
        insert_args(class_name, instance_name, results['rparen'])