from .test_all_fixers import lib3to2FixerTestCase

class Test_annotations(lib3to2FixerTestCase):
    fixer = "annotations"
    
    def test_no_annotations_1(self):
        a = 'def foo(): pass\n'
        b = a + 'foo.func_annotations = {}\n'
        
        self.check(a, b) 
    
    def test_no_annotations_2(self):
        a = """
        def foo():
            pass
            pass
        """
        b = """
        def foo():
            pass
            pass
        foo.func_annotations = {}
        """
        
        self.check(a, b)
        
    def test_no_annotations_3(self):
        a = """
        def foo(bar=baz):
            pass
            pass
        """
        b = """
        def foo(bar=baz):
            pass
            pass
        foo.func_annotations = {}
        """
        
        self.check(a, b)
        
    def test_decorator_1(self):
        a = '''
        @foo
        def bar(): pass'''
        b = '''
        @foo
        def bar(): pass
        bar.func_annotations = {}'''
        
        self.check(a, b)
    
    def test_decorator_2(self):
        a = '''
        def bar(): pass
        bar = foo(bar)'''
        
        b = '''
        def bar(): pass
        bar.func_annotations = {}
        bar = foo(bar)'''
        
        self.check(a, b)
        
    def test_decorator_3(self):
        a = '''
        def bar():
            pass
            pass
        bar = foo(bar)'''
        b = '''
        def bar():
            pass
            pass
        bar.func_annotations = {}
        bar = foo(bar)'''
        
        self.check(a, b)
        
    def test_return_annotations_alone_1(self):
        a = "def foo() -> 'bar': pass\n"
        b = "def foo(): pass\nfoo.func_annotations = {'return': 'bar'}\n"
        
        self.check(a, b)
    
    def test_return_annotations_alone_2(self):    
        a = """
        class Xyz:
            def foo() -> "bar":
                print("baz")
                print("what's next, again?")"""
        b = """
        class Xyz:
            def foo():
                print("baz")
                print("what's next, again?")
            foo.func_annotations = {'return': "bar"}"""
        
        self.check(a, b)

    def test_return_annotations_alone_3(self):    
        a = """
        class Xyz:
            def foo() -> "bar":
                print("baz")
                print("what's next, again?")
            foo = decorator(foo)"""
        b = """
        class Xyz:
            def foo():
                print("baz")
                print("what's next, again?")
            foo.func_annotations = {'return': "bar"}
            foo = decorator(foo)"""
        
        self.check(a, b)
        
    def test_single_param_annotations_1(self):
        a = "def foo(bar:'baz'): pass"
        b = "def foo(bar): pass\nfoo.func_annotations = {'bar': 'baz'}"
        
        self.check(a, b)
        
    def test_single_param_annotations_2(self):
        a = """
        def foo(bar:"baz"="spam"):
            print("what's next, again?")
            print("whatever.")
        """
        b = """
        def foo(bar="spam"):
            print("what's next, again?")
            print("whatever.")
        foo.func_annotations = {'bar': "baz"}
        """
        
        self.check(a, b)
    
    def test_varg_annotation(self):
        a = 'def foo(*varg:"bar"): pass'
        b = '''def foo(*varg): pass\nfoo.func_annotations = {'varg': "bar"}'''
        
        self.check(a, b)
    
    def test_kwargs_annotation(self):
        a = 'def foo(**kwargs:"bar"): pass'
        b = '''def foo(**kwargs): pass\nfoo.func_annotations = {'kwargs': "bar"}'''
        
        self.check(a, b)
        
    def test_multiple_param_annotations_1(self):
        a = "def foo(bar:'spam'=False, baz:'eggs'=True, ham:False='spaghetti'): pass"
        b = "def foo(bar=False, baz=True, ham='spaghetti'): pass"
        b += "\nfoo.func_annotations = {'bar': 'spam', 'baz': 'eggs', 'ham': False}"
        
        self.check(a, b)
    
    def test_multiple_param_annotations_2(self):
        a = """
        def foo(bar:"spam"=False, baz:"eggs"=True, ham:False="spam"):
            print("this is filler, just doing a suite")
            print("suites require multiple lines.")
        """
        b = """
        def foo(bar=False, baz=True, ham="spam"):
            print("this is filler, just doing a suite")
            print("suites require multiple lines.")
        foo.func_annotations = {'bar': "spam", 'baz': "eggs", 'ham': False}
        """
        
        self.check(a, b)
        
    def test_mixed_annotations_1(self):
        a = "def foo(bar=False, baz:'eggs'=True, ham:False='spaghetti') -> 'zombies': pass"
        b = "def foo(bar=False, baz=True, ham='spaghetti'): pass"
        b += "\nfoo.func_annotations = {'baz': 'eggs', 'ham': False, 'return': 'zombies'}"
        
        self.check(a, b)
    
    def test_mixed_annotations_2(self):    
        a = """
        def foo(bar:"spam"=False, baz=True, ham:False="spam") -> 'air':
            print("this is filler, just doing a suite")
            print("suites require multiple lines.")
        """
        b = """
        def foo(bar=False, baz=True, ham="spam"):
            print("this is filler, just doing a suite")
            print("suites require multiple lines.")
        foo.func_annotations = {'bar': "spam", 'ham': False, 'return': 'air'}
        """
        
        self.check(a, b)
    
    
    def test_mixed_annotations_3(self):    
        a = "def foo(bar) -> 'brains': pass"
        b = "def foo(bar): pass\nfoo.func_annotations = {'return': 'brains'}"
        
        self.check(a, b)
