from lib3to2.tests.test_all_fixers import lib3to2FixerTestCase
from unittest.case import TestCase

from ..fixes.fix_format_numbering import (
  fix_formatstring as _fix_formatstring)

class Test_format_numbering_template(TestCase):
    
    def check(self, a, b):
        _, a = _fix_formatstring(a, 0)
        self.assertEqual(a, b)
    
    def test_single(self):
        self.check("{}", "{0}")
        
    def test_single_specifier(self):
        self.check("{:8}", "{0:8}")
        
    def test_single_specifier_nested(self):
        self.check('{:{}}', '{0:{1}}')
        
    def test_single_conversion(self):
        self.check("{!r}", "{0!r}")
        
    def test_single_escaped(self):
        self.check("{{}}", "{{}}")
        
    def test_single_escaped_2(self):
        self.check("{{{}}}", "{{{0}}}")
    
    def test_multi(self):
        self.check("{}{}", "{0}{1}")
        
    def test_multi_with_keyword(self):
        self.check("{}{}{xyz}", "{0}{1}{xyz}")
    
    def test_multi_specifier_nested(self):
        self.check('{:{}}{}', '{0:{1}}{2}')
            
class Test_format_numbering_fixer(lib3to2FixerTestCase):
    fixer = "format_numbering"
    
    def test_str(self):
        a = "'{}'.format(foo)"
        b = "'{0}'.format(foo)"
        
        self.check(a, b)
    
    def test_str_paren(self):
        a = "('{}').format(foo)"
        b = "('{0}').format(foo)"
        
        self.check(a, b)
    
    def test_str_paren_multiline(self):
        a = """
        ('Line 1: {}'
        'Line 2: {}').format(foo)"""
        b = """
        ('Line 1: {0}'
        'Line 2: {1}').format(foo)"""
        
        self.check(a, b)
    
    def test_str_fun_arg_multiline(self):
        a = """
        foo('Line 1: {}'
            'Line 2: {}'.format())"""
        b = """
        foo('Line 1: {0}'
            'Line 2: {1}'.format())"""
        
        self.check(a, b)
    
    def test_str_triple(self):
        a = """'''Line 1: {}
        Line 2: {}'''.format(foo)"""
        b = '''Line 1: {0}
        Line 2: {1}'''
        b = repr(b)+'.format(foo)'
        
        self.check(a, b)
    
    def test_simple_bind(self):
        a = """
        abc = 'xyz'
        abc = '{}'
        abc.format(foo)"""
        b = """
        abc = 'xyz'
        abc = '{0}'
        abc.format(foo)"""
        
        self.check(a, b)
    
    def test_simple_bind_recursive(self):
        a = """
        abc = abc.format(foo)
        """
        
        self.unchanged(a, ignore_warnings=True)
    
    def test_simple_bind_suite(self):
        a = """
        def foo():
            abc = 'xyz'
            abc = '{}'
            abc.format(foo)"""
        b = """
        def foo():
            abc = 'xyz'
            abc = '{0}'
            abc.format(foo)"""
        
        self.check(a, b)
        
    def test_complex_bind(self):
        a = """
        if True:
            abc = '{}'
        abc.format(foo)
        """
        
        self.unchanged(a, ignore_warnings=True)