from lib2to3 import patcomp
from lib2to3.tests.test_util import parse

from unittest.case import TestCase

from ..fixer_util import indentation as get_indentation
from lib3to2.fixer_util import paste_after
from lib2to3.fixer_util import find_root

class TestIndentation(TestCase):
    
    DEFAULT_PATTERN = patcomp.compile_pattern("'TEST'")
    
    def check(self, code, indentation, pattern=DEFAULT_PATTERN):
        matches = filter(pattern.match, parse(code).pre_order())
        for node in matches:
            self.assertEqual(get_indentation(node), indentation)
    
    def test_root(self):
        a = "TEST"
        
        self.check(a, '')
        
    def test_suite_1(self):
        a = '''
        class Xyz:
            TEST'''
        
        self.check(a, '    ')
    
    def test_suite_2(self):
        a = '''
        class Xyz:
            
            TEST'''
        
        self.check(a, '    ')
    
    def test_suite_3(self):
        a = '''
        class Xyz:
            a = 5
            TEST'''
        
        self.check(a, '    ')
    
    def test_suite_4(self):
        a = '''
        class Xyz:
            
            def foo(self):
                pass
                pass
            
            TEST'''
        
        self.check(a, '    ')
    
    def test_suite_5(self):
        a = '''
        class Xyz:
            
            def foo(self):
                pass
                pass
            TEST'''
        
        self.check(a, '    ')
    
    def test_suite_6(self):
        a = '''
        class Xyz:
            if True:
                pass
                if False:
                    pass
                    pass
            
            TEST'''
        
        self.check(a, '    ')
    
    def test_suite_7(self):
        a = '''
        class Xyz:
            if True:
                pass
                if False:
                    pass
                    pass
            TEST'''
        
        self.check(a, '    ')

class TestPasteAfter(TestCase):
    
    DEFAULT_PATTERN = "funcdef< 'def' 'AFTER' any* >"
    DEFAULT_PATTERN = patcomp.compile_pattern(DEFAULT_PATTERN)
    
    def check(self, a, c, b='SIMPLE_STMT()', pattern=DEFAULT_PATTERN):
        node = next(iter(filter(pattern.match, parse(a).pre_order())))
        paste_after(node, parse(b, 1))
        self.assertEquals(str(find_root(node)), str(parse(c)))
    
    def test_indentation_1(self):
        a = """
        class Foo:
            def AFTER(): pass
        
        x = 5
        """
        c = """
        class Foo:
            def AFTER(): pass
            SIMPLE_STMT()
        
        x = 5
        """
        
        self.check(a, c)
    
    def test_indentation_2(self):
        a = """
        class Foo:
            def AFTER():
                pass
                pass
        
        x = 5
        """
        c = """
        class Foo:
            def AFTER():
                pass
                pass
            SIMPLE_STMT()
        
        x = 5
        """
        
        self.check(a, c)
    
    def test_indentation_3(self):
        a = """
        class Foo:
            def AFTER():
                pass
                pass
                
            def xyz(): pass
        
        x = 5
        """
        c = """
        class Foo:
            def AFTER():
                pass
                pass
            SIMPLE_STMT()
            
            def xyz(): pass
        
        x = 5
        """
        
        self.check(a, c)
        
    def test_indentation_4(self):
        a = """
        class Foo:
            def AFTER():
                pass
                pass
            def xyz(): pass
        
        x = 5
        """
        c = """
        class Foo:
            def AFTER():
                pass
                pass
            SIMPLE_STMT()
            def xyz(): pass
        
        x = 5
        """
        
        self.check(a, c)
    
    def test_indentation_5(self):
        a = """
        class Foo:
            def AFTER():
                if True:
                    pass
                else:
                    pass
            
            xyz = None
        
        x = 5
        """
        c = """
        class Foo:
            def AFTER():
                if True:
                    pass
                else:
                    pass
            SIMPLE_STMT()
            
            xyz = None
        
        x = 5
        """
        
        self.check(a, c)